@@ -1,4 +1,4 @@
-# $Id: Reference.pm,v 1.17 2000/06/21 19:49:40 rcaputo Exp $
+# $Id: Reference.pm,v 1.20 2000/12/26 06:14:12 rcaputo Exp $
 
 # Filter::Reference partial copyright 1998 Artur Bergman
 # <artur@vogon-solutions.com>.  Partial copyright 1999 Philip Gwyn.
@@ -154,11 +154,11 @@ __END__
 
 =head1 NAME
 
-POE::Filter::Reference - POE Freeze/Thaw Protocol Abstraction
+POE::Filter::Reference - freeze data for sending; thaw data when it arrives
 
 =head1 SYNOPSIS
 
-  $filter = new POE::Filter::Reference();
+  $filter = POE::Filter::Reference->new();
   $arrayref_of_perl_references =
     $filter->get($arrayref_of_raw_chunks_from_driver);
   $arrayref_of_serialized_perl_references =
@@ -166,31 +166,59 @@ POE::Filter::Reference - POE Freeze/Thaw Protocol Abstraction
 
 =head1 DESCRIPTION
 
-The "put" half of this filter freezes referenced Perl structures into
-serialized versions for sending.  The "get" half of this filter thaws
-serialized Perl structures back into references.  This provides a
-handy way to ship data between processes and systems.
-
-Serializers should recognize that POE::Filter::Reference is used to
-ship data between systems with different byte orders.
+This filter packages referenced data for writing to a file or socket.
+Upon receipt of packaged data, it reconstitutes the original structure
+and returns a reference to it.  This provides a handy way to ship data
+between processes and systems.
 
 =head1 PUBLIC FILTER METHODS
 
-=over 4
+=over 2
 
-=item *
+=item new SERIALIZER, COMPRESSION
 
-POE::Filter::Reference::new( ... )
+=item new SERIALIZER
 
-The new() method creates and initializes the reference filter.  It
-accepts optional parameters to specify a serializer and the use of
-compression.  The serializer may be a package or an object; the
-compression flag is a Perl "boolean" value.
+=item new
 
-A package serializer must have a thaw() function, and it must have
-either a freeze() or nfreeze() function.  If it has both freeze() and
-nfreeze(), then Filter::Reference will use nfreeze() for portability.
-These functions match Storable and FreezeThaw's call signatures.
+new() creates and initializes a reference filter.  It accepts two
+optional parameters: A serializer and a flag that determines whether
+Compress::Zlib will be used to compress serialized data.
+
+Serializers are modelled after Storable.  Storable has a nfreeze()
+function which translates referenced data into strings suitable for
+shipping across sockets.  It also contains a freeze() method which is
+less desirable since it doesn't take network byte ordering into
+effect.  Finally there's thaw() which translates frozen strings back
+into data.
+
+SERIALIZER may be a package name or an object reference, or it may be
+omitted altogether.
+
+If SERIALIZER is a package name, it is assumed that the package will
+have a thaw() function as well as etither an nfreeze() or a freeze()
+function.
+
+  # Use Storable explicitly, specified by package name.
+  my $filter = POE::Filter::Reference->new('Storable');
+
+If SERIALIZER is an object reference, it's assumed to have a thaw()
+method as well as either an nfreeze() or freeze() method.
+
+  # Use an object.
+  my $filter = POE::Filter::Reference->new($object);
+
+If SERIALIZER is omitted or undef, the Reference filter will try to
+use Storable.  If storable isn't found, it will try FreezeThaw.  And
+finally, if FreezeThaw is not found, it will die.
+
+  # Use the default filter (either Storable or FreezeThaw).
+  my $filter = POE::Filter::Reference->new();
+
+Filter::Reference will try to compress frozen strings and uncompress
+them before thawing if COMPRESSION is true.  It uses Compress::Zlib
+for this, but it works fine even without Zlib as long as COMPRESSION
+is false.
 
 An object serializer must have a thaw() method.  It also must have
 either a freeze() or nfreeze() method.  If it has both freeze() and
@@ -207,19 +235,13 @@ naming a serializer.
 For example:
 
   # Use the default filter (either Storable or FreezeThaw).
-  my $filter = new POE::Filter::Reference();
-
-  # Use Storable explicitly, specified by package name.
-  my $filter = new POE::Filter::Reference('Storable');
-
-  # Use an object.
-  my $filter = new POE::Filter::Reference($object);
+  my $filter = POE::Filter::Reference->new();
 
   # Use an object, with compression.
-  my $filter = new POE::Filter::Reference($object, 1);
+  my $filter = POE::Filter::Reference->new($object, 1);
 
   # Use the default serializer, with compression.
-  my $filter = new POE::Filter::Reference(undef, 1);
+  my $filter = POE::Filter::Reference->new(undef, 1);
 
 The new() method will try to require any packages it needs.
 
@@ -248,19 +270,23 @@ streamable representations.
 
 =head1 SEE ALSO
 
-POE::Filter; POE::Filter::HTTPD; POE::Filter::Line;
-POE::Filter::Stream
+POE::Filter.
+
+The SEE ALSO section in L<POE> contains a table of contents covering
+the entire POE distribution.
 
 =head1 BUGS
 
-Oh, probably some.
+Whatever is used to freeze and thaw data should be aware of potential
+differences in system byte orders.  Also be careful that the same
+freeze/thaw code is used on both sides of a socket.  That includes
+even the most minor version differences.
 
 =head1 AUTHORS & COPYRIGHTS
 
 The Reference filter was contributed by Arturn Bergman, with changes
 by Philip Gwyn.
 
-Please see the POE manpage for more information about authors and
-contributors.
+Please see L<POE> for more information about authors and contributors.
 
 =cut
